#include "LinearSpiral.h"
#include <math.h>
#include <iostream>
#include <sstream>

LinearSpiral::LinearSpiral()
{
    //ctor

    m_sradius = 1;
    m_rate = 1;
    m_orientation = true;
}

LinearSpiral::~LinearSpiral()
{
    //dtor
}




void LinearSpiral::setParameters(float a, float b, bool r)
{
    m_sradius = a;
    m_rate = b;
    m_orientation = r;
}


float LinearSpiral::ComputeBest(sf::Vector2f c, std::vector<pt_datum> v)
{
    if(v.size() >= 2) {
    std::vector<float> X, Y;

    float s_angle = angle(c, v[1].p)/PI*0.5;

    float xbar=0, ybar=0;
    for(int i = 0; i < v.size() ; i++)
    {
        X.push_back(angle(c, v[i].p)/PI*0.5+v[i].loops);
        Y.push_back(dist(c,v[i].p));

        xbar += X.back();
        ybar += Y.back();
    }

    xbar /= v.size();
    ybar /= v.size();

    float var_x = 0, var_y=0, cov = 0;
    for(int i = 0; i < v.size() ; i++)
    {
        var_x += (X[i] - xbar)*(X[i] - xbar);
        var_y += (Y[i] - ybar)*(Y[i] - ybar);
        cov += (X[i] - xbar)*(Y[i] - ybar);
    }
    var_x /= v.size();
    var_y /= v.size();
    cov /= v.size();

    m_rate = cov/var_x;
    m_sradius = ybar- m_rate * xbar;

    m_fit = cov/sqrt(var_x*var_y);

    /*float xbar=0, ybar=0;
    for(int i = 0; i < v.size() ; i++)
    {
        xbar += v[i].x;
        ybar += v[i].y;
    }

    xbar /= v.size();
    ybar /= v.size();

    float var_x = 0, var_y=0, cov = 0;
    for(int i = 0; i < v.size() ; i++)
    {
        var_x += (v[i].x - xbar)*(v[i].x - xbar);
        var_y += (v[i].y - ybar)*(v[i].y - ybar);
        cov += (v[i].x - xbar)*(v[i].y - ybar);
    }
    var_x /= v.size();
    var_y /= v.size();
    cov /= v.size();

    m_rate = cov/var_x;
    m_sradius = ybar- m_rate * xbar;

    m_fit = cov/sqrt(var_x*var_y);
    std::cout<<"Fit : "<<m_fit<<std::endl;*/

    }

    return m_fit;
}



sf::Vector2f LinearSpiral::ComputeBestCenter(sf::Vector2f c, std::vector<pt_datum> v)
{
    float best_fit = 0;
    sf::Vector2f best_c = c;

    for(int x = c.x - PREC_CENTER ; x <= c.x + PREC_CENTER ; x++)
    for(int y = c.y - PREC_CENTER ; y <= c.y + PREC_CENTER ; y++)
    {
        float b = ComputeBest({.x=x,.y=y},v);
        if(abs(b) > best_fit){
            best_fit = abs(b), best_c = {.x=x,.y=y};
        }
    }
    return best_c;
}



void LinearSpiral::Draw(sf::RenderTarget *W, sf::Vector2f p,  sf::Color c)
{
    sf::Vector2f curPoint;

    int mini = -10 *PREC;

    for(int i = mini; i <= 10*PREC ; i++)
   // if((m_sradius + m_rate * (float)(i-1) / (float)PREC) >= 0)
    {
        sf::Vector2f nextPoint;
        nextPoint.x = p.x + (m_sradius + m_rate * (float)i / (float)PREC) * cos((float)i*2*PI/(float)PREC) * (2*m_orientation-1);
        nextPoint.y = p.y + (m_sradius + m_rate * (float)i / (float)PREC) * sin((float)i*2*PI/(float)PREC);
        if(i != mini && (m_sradius + m_rate * (float)(i-1) / (float)PREC) >= 0)
            drawLine(W,curPoint, nextPoint,c);
        curPoint = nextPoint;
    }
}


void LinearSpiral::DrawDot(sf::RenderTarget *W, sf::Vector2f p, float a,sf::Color c)
{
    sf::Vector2f dot;
    dot.x = p.x + (m_sradius + m_rate * a * 0.5 / PI ) * cos(a) * (2*m_orientation-1);
    dot.y = p.y + (m_sradius + m_rate * a * 0.5 / PI) * sin(a);

    drawDot(W,dot,3,c);
}


void LinearSpiral::DrawArc(sf::RenderTarget *W, sf::Vector2f p, float s, float a,sf::Color c)
{
     sf::Vector2f curPoint;

     int ma = PREC*(abs(a-s)+1);

    for(int i = 0; i <= ma ; i++)
   // if((m_sradius + m_rate * (float)(i-1) / (float)PREC) >= 0)
    {
        sf::Vector2f nextPoint;
        nextPoint.x = p.x + (m_sradius + m_rate * (s+(float)i*(a-s)/(float)ma)) * cos(2*PI*(s+(float)i*(a-s)/(float)ma)) * (2*m_orientation-1);
        nextPoint.y = p.y + (m_sradius + m_rate * (s+(float)i*(a-s)/(float)ma)) * sin(2*PI*(s+(float)i*(a-s)/(float)ma));
        if(i > 0 && (m_sradius + m_rate * (s+(float)(i-1)*(a-s)/(float)ma)) >= 0)
            drawLine(W,curPoint, nextPoint,c);
        curPoint = nextPoint;
    }
}

std::string LinearSpiral::printValues()
{
    std::ostringstream  buf;
    buf<<"r = "<<m_sradius<<" + "<<m_rate*0.5/PI<<"t, ["<<abs(m_fit)*100<<"%]";
    return buf.str();
}

