#include "LogSpirale.h"
#include <math.h>
#include <iostream>
#include <sstream>




LogSpiral::LogSpiral()
{
    //ctor

    m_sradius = 1;
    m_rate = 1;
    m_orientation = true;
}

LogSpiral::~LogSpiral()
{
    //dtor
}


void LogSpiral::setParameters(float a, float b, bool r)
{
    m_sradius = a;
    m_rate = b;
    m_orientation = r;
}

void LogSpiral::setRate(float r)
{
    m_rate = r;
}

float LogSpiral::getRate()
 {
    return (m_rate);
 }

float LogSpiral::ComputeBest(sf::Vector2f c, std::vector<pt_datum> v)
{
    if(v.size() >= 2){

    std::vector<float> X, Y;

    float s_angle = angle(c, v[1].p)/PI*0.5;

    float xbar=0, ybar=0;
    for(int i = 0; i < v.size() ; i++)
    {
        X.push_back(angle(c, v[i].p)/PI*0.5+v[i].loops);
        Y.push_back(log(dist(c,v[i].p)));

        xbar += X.back();
        ybar += Y.back();
    }

    xbar /= v.size();
    ybar /= v.size();

    float var_x = 0, var_y=0, cov = 0;
    for(int i = 0; i < v.size() ; i++)
    {
        var_x += (X[i] - xbar)*(X[i] - xbar);
        var_y += (Y[i] - ybar)*(Y[i] - ybar);
        cov += (X[i] - xbar)*(Y[i] - ybar);
    }
    var_x /= v.size();
    var_y /= v.size();
    cov /= v.size();

    m_rate = cov/var_x;
    m_sradius = exp(ybar- m_rate * xbar);

    m_fit = cov/sqrt(var_x*var_y);

    /*float xbar=0, ybar=0;
    for(int i = 0; i < v.size() ; i++)
    {
        xbar += v[i].x;
        ybar += log(v[i].y);
    }

    xbar /= v.size();
    ybar /= v.size();

    float var_x = 0, var_y=0, cov = 0;
    for(int i = 0; i < v.size() ; i++)
    {
        var_x += (v[i].x - xbar)*(v[i].x - xbar);
        var_y += (log(v[i].y) - ybar)*(log(v[i].y) - ybar);
        cov += (v[i].x - xbar)*(log(v[i].y) - ybar);
    }
    var_x /= v.size();
    var_y /= v.size();
    cov /= v.size();

    m_rate = cov/var_x;
    m_sradius = exp(ybar- m_rate * xbar);

    m_fit = cov/sqrt(var_x*var_y);
    std::cout<<"Fit : "<<m_fit<<std::endl;*/

    }

    return m_fit;
}




sf::Vector2f LogSpiral::ComputeBestCenter(sf::Vector2f c, std::vector<pt_datum> v)
{
    float best_fit = 0;
    sf::Vector2f best_c = c;

    for(int x = c.x - PREC_CENTER ; x <= c.x + PREC_CENTER ; x++)
    for(int y = c.y - PREC_CENTER ; y <= c.y + PREC_CENTER ; y++)
    {
        float b = ComputeBest({.x=x,.y=y},v);
        if(abs(b) > best_fit){
            best_fit = abs(b), best_c = {.x=x,.y=y};
        }
    }
    return best_c;
}



void LogSpiral::Draw(sf::RenderTarget *W, sf::Vector2f p, sf::Color c)
{
    sf::Vector2f curPoint;

    int mini = -10 *PREC;
    for(int i = mini; i <= 10* PREC ; i++)
    {
        sf::Vector2f nextPoint;
        nextPoint.x = p.x + (m_sradius * (float)exp(m_rate * (float)i / PREC)) * cos((float)i*2*PI/(float)PREC/*+angle*/) * (2*m_orientation-1);
        nextPoint.y = p.y + (m_sradius * (float)exp(m_rate * (float)i / PREC)) * sin((float)i*2*PI/(float)PREC/*+angle*/);
        if(i != mini)
        drawLine(W,curPoint, nextPoint,c);
        curPoint = nextPoint;
    }
}



void LogSpiral::DrawDot(sf::RenderTarget *W, sf::Vector2f p,  float a,sf::Color c)
{
    sf::Vector2f dot;
    dot.x = p.x + (m_sradius * exp(m_rate * (a) * 0.5 / PI )) * cos(a) * (2*m_orientation-1);
    dot.y = p.y + (m_sradius * exp(m_rate * (a) * 0.5 / PI)) * sin(a);

    drawDot(W,dot,3,c);
}


std::string LogSpiral::printValues()
{
    std::ostringstream  buf;
    buf<<"r = "<<m_sradius<<"e^("<<m_rate*0.5/PI<<"t), ["<<abs(m_fit)*100<<"%]";
    return buf.str();
}

